/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation.
 * See the COPYING file for more information.
 *
 **********************************************************************/


#ifndef GEOS_OPPOLYGONIZE_H
#define GEOS_OPPOLYGONIZE_H

namespace geos {
namespace operation { // geos.operation
namespace polygonize { // geos.operation.polygonize

} // namespace geos.operation.polygonize
} // namespace geos.operation
} // namespace geos

//#include <geos/operation/polygonize/EdgeRing.h>
//#include <geos/operation/polygonize/PolygonizeDirectedEdge.h>
//#include <geos/operation/polygonize/PolygonizeEdge.h>
//#include <geos/operation/polygonize/PolygonizeGraph.h>
#include <geos/operation/polygonize/Polygonizer.h>

#endif

