/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2006 Refractions Research Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation.
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_OPLINEMERGE_H
#define GEOS_OPLINEMERGE_H

namespace geos {
namespace operation { // geos::operation

/// Line merging package
namespace linemerge { // geos::operation::linemerge

} // namespace geos::operation::linemerge
} // namespace geos::operation
} // namespace geos

//#include <geos/operation/linemerge/EdgeString.h>
//#include <geos/operation/linemerge/LineMergeDirectedEdge.h>
//#include <geos/operation/linemerge/LineMergeEdge.h>
//#include <geos/operation/linemerge/LineMergeGraph.h>
#include <geos/operation/linemerge/LineMerger.h>
//#include <geos/operation/linemerge/LineSequencer.h>

#endif

