/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2006 Refractions Research Inc.
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation.
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_INDEXINTERVALRTREE_H
#define GEOS_INDEXINTERVALRTREE_H

namespace geos {
namespace index { // geos.index

/** \brief
 * Contains classes that implement a static index on a set of 1-dimensional intervals,
 * using an R-Tree packed based on the order of the interval midpoints.
 */
namespace intervalrtree { // geos.index.intervalrtree

} // namespace geos.index.intervalrtree
} // namespace geos.index
} // namespace geos

#endif //GEOS_INDEXINTERVALRTREE_H

